###############################################################################
#
# <p>Rakuten Web Service Ƹ</p>
# <pre>
#   {{rakutenbookssearch ,b,5,l}}
# </pre>
# <p>
#   word
#   genre
#    a:Blended
#    b:Books
#    c:CD
#    d:DVD
#    s:Software
#    f:Foreign books
#    g:Game
#    m:Magazine
#   number
#    default:1
#    max:30
#   imagesize
#    s,m,l
#    default:s
# </p>
#
###############################################################################
package plugin::rakutensearch::RakutenBooksSearch;
use LWP::UserAgent;
use XML::Simple;
use Unicode::RecursiveDowngrade;
use Unicode::Japanese;
use Encode qw/from_to/;

#==============================================================================
# 󥹥ȥ饯
#==============================================================================
sub new {
  my $class = shift;
  my $self = {};
  return bless $self,$class;
}
#==============================================================================
# ѥ饰ե᥽å
#==============================================================================
sub paragraph {
  my $self = shift;
  my $wiki = shift;
  my $word = shift;
  my $opt = shift;
  my $num = shift;
  my $size = shift;

  my $buf = "<div class=\"rakutensearch\">";
  $buf .= &search($word,$opt,$num,$size);
  $buf .= "</div>";

  return $buf;
}


sub search {
  my $word = shift;
  my $opt = shift;
  my $num = shift;
  my $size = shift;

  my $xml;

  my %genre = (
    "a" => '000',
    "b" => '001',
    "c" => '002',
    "d" => '003',
    "s" => '004',
    "f" => '005',
    "g" => '006',
    "m" => '007',
  );

  my %imagesize = (
    "s" => 'smallImageUrl',
    "m" => 'mediumImageUrl',
    "l" => 'largeImageUrl',
    );

  my $ApplicationId = '647e54c61961afc5a7ef5611355c2ee4';
  my $AffiliateId = '1b385ec5.69744bcc.1b385ec6.83852639';
  
  from_to($word,"euc-jp","utf8");
  $word =~ s/\'/sprintf("%%%02X",ord($1))/seg;

  if(($opt eq "") || !(exists($genre{$opt}))){
    $opt = "a";
  }
  my $genreid = $genre{$opt};

  if($num eq ""){
    $num = 1;
  }

  if(($size eq "") || !(exists($imagesize{$size}))){
    $size = "s";
  }
  $size = $imagesize{$size};

  my $url = "https://app.rakuten.co.jp/services/api/BooksTotal/Search/20170404?applicationId=$ApplicationId&affiliateId=$AffiliateId&booksGenreId=$genreid&keyword=$word&format=xml&field=0";

  my $ua = LWP::UserAgent->new;
  my $res = $ua->get($url);
  my $content = $res->content;
  
  if($res->is_success){
    my $rd = Unicode::RecursiveDowngrade->new;
    $rd->filter(sub {Unicode::Japanese->new(shift,'utf8')->euc});
    $xml = $rd->downgrade(XMLin($content, SuppressEmpty => '', ForceArray => ['Items', 'Item']));
  }

  my $buf = "<table>";

  my $items = $xml->{Items}->[0]{Item};

  if($items){
    $count = 1;
    for my $item (@{$items}){
      my $title = $item->{title};
      my $image_url = $item->{$size};
      my $isbn = $item->{isbn};
      my $jan = $item->{jan};
      my $detail = $item->{affiliateUrl};
      my $publisher = $item->{publisherName};
      my $author = $item->{author};
      my $label = $item->{label};
      my $artist = $item->{artistName};
      my $hardware = $item->{hardware};
      my $os = $item->{os};

      my $other_image_url = "./plugin/rakutensearch/no_image.jpg";

      $buf .= "<tr><td rowspan=\"5\">";
      if($image_url){
        $buf .= "<a href=\"$detail\"><img src=\"$image_url\"></a>";
      }else{
        $buf .= "<a href=\"$detail\"><img src=\"$other_image_url\"></a>";
      }
      $buf .= "</td><td></td></tr><tr><td><a href=\"$detail\">$title</a></td></tr>";
      if($opt eq "a"){
        $buf .="<tr><td></td></tr><tr><td></td></tr>";
      }
      if(($opt eq "b") || ($opt eq "f") || ($opt eq "m")){
        $buf .= "<tr><td>$publisher</td></tr>";
        $buf .= "<tr><td>$author</td></tr>";
      }
      if(($opt eq "c") || ($opt eq "d")){
        $buf .= "<tr><td>$label</td></tr>";
        $buf .= "<tr><td>$artist</td></tr>";
      }
      if($opt eq "g"){
        $buf .= "<tr><td>$label</td></tr>";
        $buf .= "<tr><td>$hardware</td></tr>";
      }
      if($opt eq "s"){
        $buf .= "<tr><td>$label</td></tr>";
        $buf .= "<tr><td>$os</td></tr>";
      }
      $buf .= "<tr><td><a href=\"$detail\">ŷ</a>";
      if($isbn){
        my $isbn10 = &isbn13to10($isbn);
        $buf .= " <a href=\"http://www.amazon.co.jp/dp/".$isbn10."\">Amazon</a>";
      } elsif ($jan){
        $buf .= " <a href=\"http://www.amazon.co.jp/".$jan."/s?k=".$jan."\">Amazon</a>";
      }
      $buf .= "</td></tr>";

      if(($count >= $num) || ($count >= 30)){
        last;
      }
      $count++;
    }
    $buf .= "</table>";
  }else{
    $buf .= "<p><span class=\"error\">No Item.</span></p>";
    $buf .= "</table>";
  }

  return $buf;
}

sub isbn13to10 {
  my $isbn13 = shift;

  my $cd = 0;
  for my $i (3..11) {
    my $c = substr($isbn13, $i, 1);
    $cd += $c * (10 - ($i - 3));
  }
  $cd = $cd % 11;
  $cd = 11 - $cd;
  if ($cd == 10) {
    $cd = "X";
  } elsif ($cd == 11) {
    $cd = 0;
  }

  my $isbn10 = substr($isbn13, 3, 9).$cd;

  return $isbn10;
}

1;
